# Introduction

  - who is this manual for?
  - what is Subplot meant for?
  - who is Subplot meant for?
  - history of Subplot
  - public use cases of Subplot

# Terminology

In Subplot we use the following terminology:

* A **need** (or "want" or "ask" or "request"): what a _stakeholder_
  wants from the system at the level they are concerned. This is often
  fuzzy, vague, and unclear and there may not be any way to verify
  that is is met.

  This is what a client tells a salesperson during the sales process,
  or an open source user says in a wishlist bug report.
  
  For example: a client might ask for the backup program to
  de-duplicate data in the backups; this is too vague to be
  implemented with any confidence. It leaves open too many details:
  what kind of data should be de-duplicated? At what granularity?
  Should it happen for data across users, and if so, how much can
  users know about each others' data? Should the de-duplication happen
  on the client machines, on the server, or only in storage?

* **Acceptance criteria:** needs expressed in a way that are clear,
  unambiguous, and specific, and with a specified way for verifying
  the criteria are met.

  This is what is nailed down in the contract so that it's clear when
  payment is due; or what an open source project commits to
  supporting.

* **Stakeholder:** anyone whose opinion about the system matters and
  who gets to affect what the acceptance criteria are, for their level of
  involvement. Stakeholders can be involved at various levels in a
  project.

* **Requirements:** since acceptance criteria are about the
  stakeholders accepting the software that gets offered to them, the
  development organization may use these criteria as a basis from
  which to derive more granular and specific statements about what the
  project should do and how to check that it does so. These are the
  traditional requirements and unit/integration tests which most
  software developers work with.
 

# An overview of acceptance criteria and their verification

## Testing vs verification

"Testing software" means different things for different people. For
some, it means "finding problems", and for others it means "making
sure it works as expected". Both of these can be very open-ended and
leave unclear when one has done enough testing. For Subplot we avoid
the term "testing" and instead have a different approach: Subplot is
for documenting acceptance criteria and how to verify the software or
system under test meets them. Subplot does not try to find ways of
finding new problems.

## Functional vs non-functional requirements

A functional requirement is concerned about what the software does,
where a non-functional one is about other aspects. A functional
requirement might be "the software adds numbers together correctly",
where a non-function one might be "the software outputs the result
using a pleasing font". A functional requirement can often be
expressed in hard numbers, where a non-functional one sometimes can't.
  
## Automated vs manual verification

The ideal is to have an automated way to verify that an acceptance
criterion is met. Automation tends to lead to fewer mistakes in the
verification: computers are really good at doing the same thing
repeatedly, whereas humans are really bad at it, and get tired,
distracted, and make mistakes.

However, all verification can't be automated. For example, what is a
pleasing font? It's possible to automate the verification that a
specific font is used, but not that it is pleasing. Worse, what is
pleasing varies from person to person, and from situation to
situation.

## Stakeholders

For Subplot, a stakeholder is anyone whose opinion matters for
deciding if the software or system being considered is acceptable.
Each project has to decide who the stakeholders are.

Different stakeholders can consider the software or system at
different levels. While the levels will vary between projects, they
might be something like:
  
  * very high level: I am the CEO of the client, and I have final say
    on what we're willing to pay for, and I set the acceptance
    criteria that the system can back up all our important files on
    production systems; this will be verified by backing up all our
    production data.

  * medium level: I am the CTO of the client and I add the following
    acceptance criteria:
    
    - backed up data can be restored, verified by restoring a backup
      and comparing to the original data
    - backups are encrypted, verified by checking the backup for
      cleartext data
    - a full backup and restore of a terabyte of data takes at most
      one hour in a setup similar to our intended production systems,
      verified by measuring the backup and restore of production data

  * low level: I am the security analyst at the client and I add the
    acceptance criteria that the encryption uses SHA3 for integrity
    checking and the Ed25519 elliptic curve for encryption and digital
    signatures; these will be verified by third party inspection

## Direct vs indirect verification

The best kind of verification is direct: if the acceptance criterion
says that foo must contain bar, then the verification should check
that foo does, indeed, contain bar and nothing else.

Sometimes direct verification isn't feasible: either foo is really
hard to inspect, or it takes too long, or for some other reason. Then
an indirect verification can be used. If we can't see everything
that's inside of foo, but we can check if there is a specific letter
inside, we can do an indirect verification that foo contains the
letters b, a, and r. This doesn't verify that foo contains bar: it
might contain arb instead. Also, since there are a nearly infinite
number of possible letter, maybe foo actually contains xyzzy.

How directly we verify depends on a variety of things, most
importantly how much time and effort we can put into creating the
verification method, or into executing it.

Also, it matters what the consequences are if the verification falsely
accepts a faulty system. If the health or life of someone is
concerned, the verification must be done much more carefully.

## Verification by scenario

Subplot uses a verification approach based on scenarios, inspired by
the [Cucumber](https://en.wikipedia.org/wiki/Cucumber_(software))
system.. Each scenario consists of steps:

* a "given" step sets up the verification
* a "when" step performs an action the acceptance criterion is
  concerned about
* a "then" step verifies that the result of the action is as desired

There can be many steps of each and they do not have to be in any
particular order. The two goals of the scenario is to document the
steps to verify that an acceptance criterion is met, and to allow
Subplot to generate code to automatically perform the verification.

For example, the following scenario verifies that a file can be
compress and uncompressed without altering its contents.

* given a file foo.dat with some random data
* when I compress foo.dat into compressed.dat
* when I uncompress compressed.dat into bar.dat
* then files foo.dat and bar.dat are identical

This should be clear to any human stakeholder who understands computer
files and running programs. They need to not understand how each step
is actually implemented, as long as they understand the purpose of
each step, and, crucially, trust that the actual implementation
conforms to the purpose of the step. Such trust is always going go be
needed at some level.

Subplot translates each step into a call to some code (a "function"),
and can extract parts of the step text to be an argument or parameter
given to the code. Thus, for the above scenario, Subplot might produce
the following function calls:

* `write_random_data("foo.dat")`
* `compress_file("foo.dat", "compressed.dat")`
* `uncompress_file("compressed.dat", "bar.dat")`
* `compare_files("foo.dat", "bar.dat")`

The steps that are available to use in any given Subplot document are
defined for that document in one or more "binding" files. We'll get to
the details of that later. At this stage, the important thing is that
scenario steps are defined for each project, and are meant to be
building blocks that can be combined in easy, obvious ways to
formulate verification scenarios such that the scenario makes sense
for the stakeholders, and the generated code actually verifies what
the stakeholders think it does.

It is important that all stakeholders understand scenarios aimed at
their level of participation. The stakeholders need to agrees that the
acceptance criteria, taken together, capture enough of their concerns
about the software or system under discussion that they will accept it
if the criteria are met. They also need to give informed agreement
that the scenarios correctly and sufficiently verify that the criteria
are met.

This is difficult. It requires writing scenarios using what is not
really natural language, but is close enough that even those not used
to software code understand them, but not so far from program code
that it gets difficult to implement the scenario steps in code.


# Simple example project

  - discuss how to use Subplot for some simple, but not simplistic,
    software project
  - discuss different kinds of stakeholders a project may have

# Authoring Subplot documents

  - discuss that it may be necessary to have several documents for
    different audiences, at different levels of abstraction (cf. the
    FOSDEM safety devroom talk)
  - discuss writing style to target all the different stakeholders
  - discuss mechanics and syntax
    - Markdown and supported features
    - scenarios, embedded files, examples
    - bindings
    - step implementations in various languages
    - embedded markup for diagrams
    - running docgen

# Extended example project

  - discuss how to use Subplot for a significant project, but keep it
    sufficiently high level that it doesn't get too long and tedious
    to read

# Appendix: Implementing scenario steps in Bash

  - this appendix will explain how to implement scenario steps using
    the Bash shell

# Appendix: Implementing scenario steps in Python

  - this appendix will explain how to implement scenario steps using
    the Python language

# Appendix: Implementing scenario steps in Rust

  - this appendix will explain how to implement scenario steps using
    the Rust language


# Appendix: Scenario

This is currently necessary so that codegen won't barf.

~~~scenario
when I run true
~~~
